package com.gic.plug.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.error.ErrorCode;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import com.gic.thirdparty.pic.QqCloudPicUtils;
import com.gic.thirdparty.pic.UploadResult;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.sanselan.ImageInfo;
import org.apache.sanselan.ImageReadException;
import org.apache.sanselan.Sanselan;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
/**
 * 上传
 * @ClassName: UploadImgController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/14 10:55 AM 
 */
@RestController
public class UploadImgController {
    private static Logger logger = LogManager.getLogger(UploadImgController.class);

    @RequestMapping("upload-images")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse uploadImages(HttpServletRequest request) throws IOException {
        MultipartHttpServletRequest multiRequest = (MultipartHttpServletRequest) request;
        Iterator<String> iter = multiRequest.getFileNames();
        java.util.List<Map<String, Object>> picList = new ArrayList<Map<String, Object>>();
        while (iter.hasNext()) {
            String fileName = iter.next();
            List<MultipartFile> list = multiRequest.getMultiFileMap().get(fileName);
            Map<String, Object> map = null;
            for (int i = 0; i < list.size(); i++) {
                MultipartFile mf = list.get(i);
                ImageInfo imageInfo = null;
                try {
                    imageInfo = Sanselan.getImageInfo(mf.getBytes());
                } catch (ImageReadException e) {
                    logger.warn("图片解析异常", e);
                }
                if(imageInfo.getColorType() == ImageInfo.COLOR_TYPE_CMYK){
                    return RestResponse.failure(ErrorCode.NOT_SUPPORT.getCode(), "图片不支持上传CMYK色彩模式，请上传RGB色彩模式的图片");
                }
                UploadResult uploadResult = QqCloudPicUtils.uploadLocalPicStream("pic01", "", mf.getBytes());
                logger.info("上传图片结果："+JSON.toJSONString(uploadResult));
                Map<String, Object> pic = new HashMap<>(8);
                pic.put("fileId", uploadResult.getFileId());
                pic.put("url", uploadResult.getUrl());
                pic.put("downloadUrl", uploadResult.getDownloadUrl());
                picList.add(pic);
            }
        }
        return RestResponse.success(picList);
    }

    @RequestMapping("get-cos-secret")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse getCosSecret(HttpServletRequest request) throws IOException {
        Map<String, String> map = new HashMap<>(8);
        map.put("secretId", "AKIDxpbnsmZ0KYtAKNd5f7QbbP3YyDJD6Rq3");
        map.put("secretKey", "GLFwRjpmYviRzLUREqKcieS4VmsmU1Lc");
        byte[] bytes = map.get("secretKey").getBytes();
        //Base64 加密
        String encoded = Base64.getEncoder().encodeToString(bytes);
        map.put("secretKey", encoded);
        map.put("bucket", "video-1251519181");
        map.put("region", "ap-shanghai");
        return RestResponse.success(map);
    }

    @RequestMapping("upload-file")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse uploadFile(HttpServletRequest request) throws IOException {
        MultipartHttpServletRequest multiRequest = (MultipartHttpServletRequest) request;
        Iterator<String> iter = multiRequest.getFileNames();
        java.util.List<Map<String, Object>> picList = new ArrayList<Map<String, Object>>();
        while (iter.hasNext()) {
            String fileName = iter.next();
            List<MultipartFile> list = multiRequest.getMultiFileMap().get(fileName);
            for (int i = 0; i < list.size(); i++) {
                MultipartFile mf = list.get(i);
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
                //获得原始文件名;
                String fileRealName = mf.getOriginalFilename();
                //点号的位置
                int pointIndex =  fileRealName.lastIndexOf(".");
                //截取文件后缀
                String fileSuffix = fileRealName.substring(pointIndex);
                logger.info("上传的文件{},后缀是：{}", fileRealName, fileSuffix);

                String fieldCode = "/" + fileName + "_" + sdf.format(new Date()) + fileSuffix;

                File targetFile = null;
                try {
                    targetFile = multipartFileToFile(mf);
                    //默认是其他
                    String bucketName = BucketNameEnum.OTHER_90000.getName();
                    if (isPicture(fileSuffix)) {
                        bucketName = BucketNameEnum.IMAGE_10000.getName();
                    } else if (isOffice(fileSuffix)) {
                        bucketName = BucketNameEnum.REPORT_50000.getName();
                    } else if (isMusic(fileSuffix)) {
                        bucketName = BucketNameEnum.MUSIC_20000.getName();
                    } else if (isVedio(fileSuffix)) {
                        bucketName = BucketNameEnum.VIDEO_40000.getName();
                    }
                    String url = FileOperateUtils.simpleUploadFileFromLocal(targetFile, fieldCode, bucketName);
                    if (StringUtils.isNotBlank(url)) {
                        url = "https://" + url;
                    }

                    Map<String, Object> pic = new HashMap<>(8);
                    pic.put("fileId", fieldCode);
                    pic.put("url", url);
                    pic.put("fileName", fileRealName);
                    picList.add(pic);
                } catch (Exception e) {
                    logger.warn("上传文件{}错误：{}", fileRealName, e.getMessage(), e);
                } finally {
                    delteTempFile(targetFile);
                }
            }
        }
        return RestResponse.success(picList);
    }

    public static File multipartFileToFile(MultipartFile file) throws Exception {
        File toFile = null;
        if ("".equals(file) || file.getSize() <= 0) {
            file = null;
        } else {
            InputStream ins = null;
            ins = file.getInputStream();
            toFile = new File(file.getOriginalFilename());
            inputStreamToFile(ins, toFile);
            ins.close();
        }
        return toFile;
    }

    /**
     * 获取流文件
     * @param ins
     * @param file
     */
    private static void inputStreamToFile(InputStream ins, File file) {
        try {
            int bufferSize = 8192;
            OutputStream os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[bufferSize];
            while ((bytesRead = ins.read(buffer, 0, bufferSize)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            ins.close();
        } catch (Exception e) {
            logger.warn("e", e);
        }
    }

    /**
     * 删除本地临时文件
     * @param file
     */
    public static void delteTempFile(File file) {
        if (file != null) {
            File del = new File(file.toURI());
            del.delete();
        }
    }

    /**
     * 是否是图片
     * @param fileSuffix
     * @return
     * @throws Exception
     */
    public static boolean isPicture(String fileSuffix) throws Exception{
        String[] imageArray =
                {"bmp", "dib","gif", "jfif", "jpe","jpeg", "jpg", "png", "tif", "tiff", "ico", "10"};
        return Arrays.stream(imageArray).anyMatch(e -> e.equals(fileSuffix.replace(".", "")));
    }

    public static boolean isOffice(String fileSuffix) throws Exception{
        String[] arr =
                {"doc", "docx","xls", "xlsx", "pptx", "ppt"};
        return Arrays.stream(arr).anyMatch(e -> e.equals(fileSuffix.replace(".", "")));
    }

    public static boolean isMusic(String fileSuffix) throws Exception{
        String[] arr =
                {"wma", "mpc","ogg", "wav", "mpeg", "mp3"};
        return Arrays.stream(arr).anyMatch(e -> e.equals(fileSuffix.replace(".", "")));
    }

    public static boolean isVedio(String fileSuffix) throws Exception{
        String[] arr =
                {"rm", "rmvb","mp4", "wmv", "asf", "asx", "3gp", "mov", "m4v", "avi", "dat", "mkv", "flv", "vob"};
        return Arrays.stream(arr).anyMatch(e -> e.equals(fileSuffix.replace(".", "")));
    }
}
