package com.gic.plug.web.strategy.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.plug.web.strategy.ScreenBackStrategy;
import com.gic.plug.web.utils.ComputeBackUtil;
import com.gic.plug.web.vo.ScreenBack;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreFieldDateEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.utils.StoreRedisKeyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
 * 门店
 * @ClassName: StoreCustomBackImpl 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/14 10:55 AM 
 */
@Component
public class StoreCustomBackImpl implements ScreenBackStrategy {
    private static Logger log = LogManager.getLogger(StoreCustomBackImpl.class);
    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;
    private Map<Integer, String> methodMap = new HashMap<>();

    @PostConstruct
    public void init() {
        methodMap.put(StoreFieldDateEnum.YMDHMS.getCode(), "yyyyMMddHHmmssBack");
        methodMap.put(StoreFieldDateEnum.YMD.getCode(), "yyyyMMddBack");
        methodMap.put(StoreFieldDateEnum.YM.getCode(), "yyyyMM");
        methodMap.put(StoreFieldDateEnum.MD.getCode(), "mmddBack");
        methodMap.put(StoreFieldDateEnum.Hms.getCode(), "hhmmssBack");
    }

    @Override
    public ScreenBack screenBack(String compute, String value, String key, Integer enterpriseId) {
        if (StringUtils.isNotBlank(value)) {
            Object cache = RedisUtil.getCache(StoreRedisKeyUtils.getStoreFieldKeyC(enterpriseId, key));
            if (cache != null) {
                ServiceResponse<StoreFieldDTO> storeField = this.storeFieldApiService.getStoreField(Integer.valueOf(cache.toString()));
                if (storeField.isSuccess()) {
                    if (storeField.getResult() != null) {
                        if (storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.DATE.getCode()) {
                            return this.dateBack(compute, value, storeField.getResult());
                        } else if(storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.REAL_NUMBER.getCode()){
                            return this.numberBack(compute, value, storeField.getResult());
                        } else{
                            return this.selectBack(compute, value, enterpriseId, storeField.getResult());
                        }
                    }
                }
            }
        }
        return null;
    }

    private ScreenBack dateBack(String compute, String value, StoreFieldDTO storeFieldDTO) {
        ScreenBack screenBack = new ScreenBack();
        screenBack.setTitle(storeFieldDTO.getStoreFieldName());
        screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
        JSONObject jsonObject = JSON.parseObject(storeFieldDTO.getStoreFieldDetail());
        int dateType = jsonObject.getIntValue("dateType");
        String methodName = this.methodMap.get(dateType);
        if (StringUtils.isNotBlank(methodName)) {
            try {
                try {
                    Method method = this.getClass().getMethod(methodName, Class.forName(value.getClass().getCanonicalName()));
                    try {
                        String message = (String) method.invoke(this, value);
                        List<String> view = new ArrayList<>();
                        view.add(message);
                        return screenBack;
                    } catch (IllegalAccessException e) {
                        log.warn("e", e);
                    } catch (InvocationTargetException e) {
                        log.warn("e", e);
                    }
                } catch (ClassNotFoundException e) {
                    log.warn("e", e);
                }
            } catch (NoSuchMethodException e) {
                log.warn("e", e);
            }
        }
        return null;
    }

    private ScreenBack selectBack(String compute, String value, Integer enterpriseId, StoreFieldDTO storeFieldDTO) {
        ScreenBack screenBack = new ScreenBack();
        screenBack.setTitle(storeFieldDTO.getStoreFieldName());
        screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
        List<Integer> list = new ArrayList<>();
        String[] valueArr = value.split(" ");
        for (String s : valueArr) {
            list.add(Integer.valueOf(s));
        }
        ServiceResponse<List<StoreFieldSelectDTO>> listServiceResponse = this.storeFieldSelectApiService.listStoreFieldSelectByIds(enterpriseId, list);
        if (listServiceResponse.isSuccess()) {
            List<StoreFieldSelectDTO> selectDtoS = listServiceResponse.getResult();
            List<String> view = new ArrayList<>();
            List<String> hideView = new ArrayList<>();
            int index = 0;
            if (CollectionUtils.isNotEmpty(selectDtoS)) {
                for (StoreFieldSelectDTO dto : selectDtoS) {
                    if(index < 5){
                        view.add(dto.getStoreFieldSelectName());
                    } else {
                        hideView.add(dto.getStoreFieldSelectName());
                    }
                }
            }
            screenBack.setView(view);
            screenBack.setHideView(hideView);
        }
        return screenBack;
    }

    private ScreenBack numberBack(String compute, String value, StoreFieldDTO storeFieldDTO){
        StringBuilder stringBuilder = new StringBuilder();
        ScreenBack screenBack = new ScreenBack();
        screenBack.setTitle(storeFieldDTO.getStoreFieldName());
        screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
        List<String> view = new ArrayList<>();
        if(StringUtils.isNotBlank(value)){
            String[] valueArr = value.split(" ");
            stringBuilder.append(valueArr[0]);
            if(valueArr.length > 1){
                stringBuilder.append(" 至 ").append(valueArr[1]);
            }
            view.add(stringBuilder.toString());
        }
        screenBack.setView(view);
        return screenBack;
    }

    public String yearMonthDateHourMinuteSecondBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat simpleDateFormatCommon = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return this.dateBack(value, simpleDateFormat, simpleDateFormatCommon);
    }

    public String yearMonthDateBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat simpleDateFormatCommon = new SimpleDateFormat("yyyy-MM-dd");
        return this.dateBack(value, simpleDateFormat, simpleDateFormatCommon);
    }

    public String yearMonth(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");
        SimpleDateFormat simpleDateFormatCommon = new SimpleDateFormat("yyyy-MM");
        return this.dateBack(value, simpleDateFormat, simpleDateFormatCommon);
    }

    public String monthDateBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("MMdd");
        SimpleDateFormat simpleDateFormatCommon = new SimpleDateFormat("MM-dd");
        return this.dateBack(value, simpleDateFormat, simpleDateFormatCommon);
    }

    public String hourMinuteSecondBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HHmmss");
        SimpleDateFormat simpleDateFormatCommon = new SimpleDateFormat("HH:mm:ss");
        return this.dateBack(value, simpleDateFormat, simpleDateFormatCommon);
    }

    private String dateBack(String value, SimpleDateFormat simpleDateFormat, SimpleDateFormat simpleDateFormatCommon) throws ParseException {
        String[] valueArr = value.split(" ");
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("【").append(simpleDateFormatCommon.format(simpleDateFormat.parse(valueArr[0]))).append("】").append(" ");
        if (valueArr.length > 1) {
            stringBuilder.append("至");
            stringBuilder.append("【").append(simpleDateFormatCommon.format(simpleDateFormat.parse(valueArr[1]))).append("】").append(" ");
        }
        return stringBuilder.toString();
    }
}
