package com.gic.plug.web.strategy.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.plug.web.strategy.ScreenBackStrategy;
import com.gic.plug.web.utils.ComputeBackUtil;
import com.gic.plug.web.vo.ScreenBack;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.constant.StoreCompleteEnum;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.constant.StoreStatusEnum;
import com.gic.store.dto.StoreDictDTO;
import com.gic.store.service.StoreDictApiService;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
 * 门店
 * @ClassName: StoreSelectBackImpl 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/14 10:55 AM 
 */
@Component
public class StoreSelectBackImpl implements ScreenBackStrategy {
    private static Logger log = LogManager.getLogger(StoreSelectBackImpl.class);
    @Autowired
    private StoreDictApiService storeDictApiService;
    private Map<String, String> methodMap = new HashMap<>();

    @PostConstruct
    public void init(){
        methodMap.put(StoreESFieldsEnum.STATUS.getField(), "erpStatusBack");
        methodMap.put(StoreESFieldsEnum.COMPLETESTATUS.getField(), "storeCompleteBack");
        methodMap.put(StoreESFieldsEnum.CREATETYPE.getField(), "storeCreateTypeBack");
        methodMap.put(StoreESFieldsEnum.STORETYPE.getField(), "storeTypeBack");
    }
    @Override
    public ScreenBack screenBack(String compute, String value, String key, Integer enterpriseId){
        String methodName = methodMap.get(key);
        if(StringUtils.isNotBlank(methodName)){
            try {
                try {
                    Method method = this.getClass().getMethod(methodName, Class.forName(compute.getClass().getCanonicalName()),
                            Class.forName(value.getClass().getCanonicalName()),Class.forName(enterpriseId.getClass().getCanonicalName()));
                    try {
                        Object invoke = method.invoke(this, compute, value, enterpriseId);
                        return (ScreenBack) invoke;
                    } catch (IllegalAccessException e) {
                        log.warn("e", e);
                    } catch (InvocationTargetException e) {
                        log.warn("e", e);
                    }
                } catch (ClassNotFoundException e) {
                    log.warn("e", e);
                }
            } catch (NoSuchMethodException e) {
                log.warn("e", e);
            }
        }
        return null;
    }

    /**
     * 单选
     * @param compute
     * @param value
     * @param enterpriseId
     * @return
     */
    public ScreenBack storeStatusBack(String compute, String value, Integer enterpriseId){
        if(StringUtils.isNotBlank(value)){
            ScreenBack screenBack = new ScreenBack();
            screenBack.setTitle(StoreESFieldsEnum.STATUS.getDesc());
            screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
            List<String> view = new ArrayList<>();
            for(StoreStatusEnum storeStatusEnum : StoreStatusEnum.values()){
                if(storeStatusEnum.getCode() == Integer.valueOf(value)){
                    view.add(storeStatusEnum.getMessage());
                }
            }
            screenBack.setView(view);
            screenBack.setHideView(new ArrayList<>());
            return screenBack;
        }
        return null;
    }

    /**
     * 单选
     * @param compute
     * @param value
     * @param enterpriseId
     * @return
     */
    public ScreenBack storeCompleteBack(String compute, String value, Integer enterpriseId){
        if(StringUtils.isNotBlank(value)){
            ScreenBack screenBack = new ScreenBack();
            screenBack.setTitle(StoreESFieldsEnum.COMPLETESTATUS.getDesc());
            screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
            List<String> view = new ArrayList<>();
            for(StoreCompleteEnum storeStatusEnum : StoreCompleteEnum.values()){
                if(storeStatusEnum.getCode() == Integer.valueOf(value)){
                    view.add(storeStatusEnum.getMessage());
                }
            }
            screenBack.setView(view);
            screenBack.setHideView(new ArrayList<>());
            return screenBack;
        }
        return null;
    }

    /**
     * 多选
     * @param compute
     * @param value
     * @param enterpriseId
     * @return
     */
    public ScreenBack storeCreateTypeBack(String compute, String value, Integer enterpriseId){
        if(StringUtils.isNotBlank(value)){
            ScreenBack screenBack = new ScreenBack();
            screenBack.setTitle(StoreESFieldsEnum.CREATETYPE.getDesc());
            screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
            List<String> view = new ArrayList<>();
            List<String> hideView = new ArrayList<>();
            int index = 0;
            String[] valueArr = value.split(" ");
            for(String v : valueArr){
                for(CreateTypeEnum createTypeEnum : CreateTypeEnum.values()){
                    if(createTypeEnum.getCode() == Integer.valueOf(v)){
                        if(index<5){
                            view.add(createTypeEnum.getMessage());
                        } else {
                            hideView.add(createTypeEnum.getMessage());
                        }
                    }
                }
            }
            screenBack.setView(view);
            screenBack.setHideView(hideView);
            return screenBack;
        }
        return null;
    }

    /**
     * 多选
     * @param compute
     * @param value
     * @param enterpriseId
     * @return
     */
    public ScreenBack storeTypeBack(String compute, String value, Integer enterpriseId){
        if(StringUtils.isNotBlank(value)){
            String[] valueArr = value.split(" ");
            ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreType(enterpriseId);
            if(serviceResponse.isSuccess()){
                List<StoreDictDTO> list = serviceResponse.getResult();
                ScreenBack screenBack = new ScreenBack();
                screenBack.setTitle(StoreESFieldsEnum.STORETYPE.getDesc());
                screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
                int index = 0;
                List<String> view = new ArrayList<>();
                List<String> hideView = new ArrayList<>();
                for(StoreDictDTO dictDTO : list){
                    for(String v : valueArr){
                        if(dictDTO.getValue().equals(v)){
                            if(index < 5){
                                view.add(dictDTO.getKey());
                            } else {
                                hideView.add(dictDTO.getKey());
                            }
                            index++;
                        }
                    }
                }
                screenBack.setView(view);
                screenBack.setHideView(hideView);
                return screenBack;
            }
        }
        return null;
    }

    /**
     * 多选
     * @param compute
     * @param value
     * @param enterpriseId
     * @return
     */
    public ScreenBack erpStatusBack(String compute, String value, Integer enterpriseId){
        if(StringUtils.isNotBlank(value)){
            String[] valueArr = value.split(" ");
            ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreStatus(enterpriseId);
            if(serviceResponse.isSuccess()){
                List<StoreDictDTO> list = serviceResponse.getResult();
                ScreenBack screenBack = new ScreenBack();
                screenBack.setTitle(StoreESFieldsEnum.STATUS.getDesc());
                screenBack.setCompute(ComputeBackUtil.getComputeBack(compute));
                int index = 0;
                List<String> view = new ArrayList<>();
                List<String> hideView = new ArrayList<>();
                for(StoreDictDTO dictDTO : list){
                    for(String v : valueArr){
                        if(dictDTO.getValue().equals(v)){
                            if(index < 5){
                                view.add(dictDTO.getKey());
                            } else {
                                hideView.add(dictDTO.getKey());
                            }
                        }
                    }
                }
                screenBack.setView(view);
                screenBack.setHideView(hideView);
                return screenBack;
            }
        }
        return null;
    }
}
