package com.gic.plug.web.utils;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.utils.storeresourceauth.StoreResourceUtils;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreWidgetApiService;

/**
 * 资源组-门店资源
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/14 3:56 PM 
 */
public class StoreResourceAuthUtils {

    /**
     * 获取用户的资源组-门店数据
     * @param field 门店域/门店分组......StoreESFieldsEnum
     * @param storeWidgetApiService
     * @return 空集合，表示全部权限
     */
    public static Set<String> getAuthData(String field, StoreWidgetApiService storeWidgetApiService) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        boolean isAdmin = userDetail.getUserInfo().getSuperAdmin() == 1 || userDetail.getUserResourceInfo() == null;
        //超管
        if (isAdmin) {
            return getAuthDataWhenAdmin();
        }
        Integer storeWidgetId = getStoreWidgetId();
        if (storeWidgetId == null) {
            //数据错误。正确情况管理员必有资源组数据
            return getAuthDataWhenNo();
        }
        ServiceResponse<StoreWidgetDTO> storeWidget = storeWidgetApiService.getStoreWidget(storeWidgetId);
        if (storeWidget.isSuccess()) {
            return getAuthData(storeWidget.getResult(), field);
        } else {
            //不存在
            return getAuthDataWhenNo();
        }
    }

    private static Set<String> getAuthData(StoreWidgetDTO storeWidget, String field) {
        Set<String> authRegionSet = new HashSet<>();

        String searchParam = storeWidget.getSearchParam();
        JSONObject json = JSON.parseArray(searchParam).getJSONObject(0);
        JSONArray list = json.getJSONArray("list");

        Set<String> storeFieldKey = getStoreFieldKey();

        //是否是自定义字段类型的数据
        //因为这两者解析不一致
        boolean isStoreField = isStoreField(field);

        for (int i = 0, len = list.size(); i < len; i ++) {
            JSONObject o = list.getJSONObject(i).getJSONObject("data");
            String key = o.getString("key");

            if (isStoreField) {
                //如果是自定义字段，应该存key，而不是value
                if (storeFieldKey.contains(key)) {
                    authRegionSet.add(key);
                }
                continue;
            }
            if (key.equals(field)) {
                String[] arr = o.getString("value").split(" ");
                for(String s : arr){
                    authRegionSet.add(s);
                }
            }
        }
        return authRegionSet;
    }

    /**
     * 超级管理员资源
     * @return
     */
    private static Set<String> getAuthDataWhenAdmin() {
        return  new HashSet<>(2);
    }

    /**
     * 没有资源的时候
     * @return
     */
    private static Set<String> getAuthDataWhenNo() {
        return new HashSet<>(Arrays.asList("-1"));
    }

    /**
     * 管理员资源组-门店选择器ID
     * @return
     */
    private static Integer getStoreWidgetId() {
        //todo 前端资源组对齐后，调整最新接口
        return UserDetailUtils.getUserDetail().getUserResourceInfo().getStoreResource().intValue();
    }

    private static boolean isStoreField(String field) {
        if (StoreResourceUtils.STORE_FIELD_KEY.equals(field)) {
            return true;
        }
        return false;
    }

    /**
     * 自定义字段的key
     * @return
     */
    private static Set<String> getStoreFieldKey() {
        Set<String> storeField = new HashSet<>();
        storeField.add(StoreESFieldsEnum.C1.getField());
        storeField.add(StoreESFieldsEnum.C2.getField());
        storeField.add(StoreESFieldsEnum.C3.getField());
        storeField.add(StoreESFieldsEnum.C4.getField());
        storeField.add(StoreESFieldsEnum.C5.getField());
        storeField.add(StoreESFieldsEnum.C6.getField());
        storeField.add(StoreESFieldsEnum.C7.getField());
        storeField.add(StoreESFieldsEnum.C8.getField());
        storeField.add(StoreESFieldsEnum.C9.getField());
        storeField.add(StoreESFieldsEnum.C10.getField());
        return storeField;
    }
}
