package com.gic.plug.web.utils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.qo.StoreWidgetQO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.dto.StoreWidgetLogSaveDTO;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.StoreWidgetLogApiService;

/**
 * 门店选择器操作日志工具类
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/13 7:54 PM 
 */
@Component
public class StoreWidgetLogUtils {
    @Autowired
    private StoreWidgetLogApiService storeWidgetLogApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    /**
     * 写门店选择器日志
     * @param storeWidgetQO
     * @param newStoreWidgetId
     */
    public void saveStoreWidgetLog(StoreWidgetQO storeWidgetQO, Integer newStoreWidgetId) {
        Integer storeWidgetId = storeWidgetQO.getStoreWidgetId();
        //如果是编辑，则会有值，用于判断创建还是编辑
        StoreWidgetDTO oldStoreWidget = null;

        if (storeWidgetId != null) {
            ServiceResponse<StoreWidgetDTO> oldStoreWidgetResponse = storeWidgetApiService.getStoreWidget(storeWidgetId);
            if (!oldStoreWidgetResponse.isSuccess()) {
                return;
            }
            oldStoreWidget = oldStoreWidgetResponse.getResult();
        }
        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO;

        if (oldStoreWidget == null) {
            logSaveDTO = buildStoreWidgetLogWhenCreate(storeWidgetQO, newStoreWidgetId);
        } else {
            //需要更新日志关联关系
            storeWidgetLogApiService.tmpUpdateByStoreWidgetId(storeWidgetId, newStoreWidgetId);
            logSaveDTO = buildStoreWidgetLogWhenUpdate(oldStoreWidget, storeWidgetQO, newStoreWidgetId);
        }
        //写日志
        storeWidgetLogApiService.saveStoreWidgetLog(logSaveDTO);
    }

    /**
     * 组装日志对象-编辑场景
     * @param oldStoreWidget 老数据
     * @param storeWidgetQO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenUpdate(StoreWidgetDTO oldStoreWidget, StoreWidgetQO storeWidgetQO,
                                                                      Integer newStoreWidgetId) {
        if (oldStoreWidget == null) {
            return null;
        }
        String authSearchParam = storeWidgetQO.getAuthSearchParam();
        String searchParam = storeWidgetQO.getSearchParam();

        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId);

        //编辑
        //编辑类型
        logSaveDTO.setOperationType(2);
        //是否是权限变更范围类型
        boolean isNotAuthChangeType = StringUtils.isBlank(authSearchParam) || authSearchParam.equals(oldStoreWidget.getAuthSearchParam());
        if (isNotAuthChangeType) {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);

            logSaveDTO.setSearchParamBefore(oldStoreWidget.getSearchParam());
            if (StringUtils.isNotBlank(searchParam)) {
                logSaveDTO.setSearchParamAfter(searchParam);
            }
        } else {
            //权限变更范围类型
            logSaveDTO.setLogType(1);

            logSaveDTO.setSearchParamBefore(oldStoreWidget.getAuthSearchParam());
            if (StringUtils.isNotBlank(authSearchParam)) {
                logSaveDTO.setSearchParamAfter(authSearchParam);
            }
        }
        return logSaveDTO;
    }

    /**
     * 组装日志对象-创建场景
     * @param storeWidgetQO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenCreate(StoreWidgetQO storeWidgetQO, Integer newStoreWidgetId) {
        String authSearchParam = storeWidgetQO.getAuthSearchParam();
        String searchParam = storeWidgetQO.getSearchParam();

        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId);

        //创建
        logSaveDTO.setOperationType(1);
        boolean isAuthChangeType = StringUtils.isNotBlank(authSearchParam);
        if (isAuthChangeType) {
            //权限变更范围类型
            logSaveDTO.setLogType(1);
            logSaveDTO.setSearchParamAfter(authSearchParam);
        } else {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);
            logSaveDTO.setSearchParamAfter(searchParam);
        }
        return logSaveDTO;
    }

    private static StoreWidgetLogSaveDTO buildCommon(Integer newStoreWidgetId) {
        StoreWidgetLogSaveDTO logSaveDTO = new StoreWidgetLogSaveDTO();
        logSaveDTO.setOperationId(UserDetailUtils.getUserDetail().getUserId());
        logSaveDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        logSaveDTO.setStoreWidgetId(newStoreWidgetId);
        return logSaveDTO;
    }
}
